@@ -1,14 +1,10 @@
 /******************************************************************************
- * hypercall-x86_64.h
+ * hypercall-x86_32.h
  * 
  * Copied from XenLinux.
  * 
  * Copyright (c) 2002-2004, K A Fraser
  * 
- * 64-bit updates:
- *   Benjamin Liu <benjamin.liu@intel.com>
- *   Jun Nakajima <jun.nakajima@intel.com>
- * 
  * This file may be distributed separately from the Linux kernel, or
  * incorporated into other software packages, subject to the following license:
  * 
@@ -31,11 +27,12 @@
  * IN THE SOFTWARE.
  */
 
-#ifndef __HYPERCALL_X86_64_H__
-#define __HYPERCALL_X86_64_H__
+#ifndef __HYPERCALL_X86_32_H__
+#define __HYPERCALL_X86_32_H__
 
 #include <xen/xen.h>
 #include <xen/sched.h>
+#include <xen/nmi.h>
 #include <mm.h>
 
 #define __STR(x) #x
@@ -59,7 +56,7 @@ extern char hypercall_page[PAGE_SIZE];
 	long __res, __ign1;					\
 	asm volatile (						\
 		"call hypercall_page + ("STR(__HYPERVISOR_##name)" * 32)"\
-		: "=a" (__res), "=D" (__ign1)			\
+		: "=a" (__res), "=b" (__ign1)			\
 		: "1" ((long)(a1))				\
 		: "memory" );					\
 	(type)__res;						\
@@ -70,7 +67,7 @@ extern char hypercall_page[PAGE_SIZE];
 	long __res, __ign1, __ign2;				\
 	asm volatile (						\
 		"call hypercall_page + ("STR(__HYPERVISOR_##name)" * 32)"\
-		: "=a" (__res), "=D" (__ign1), "=S" (__ign2)	\
+		: "=a" (__res), "=b" (__ign1), "=c" (__ign2)	\
 		: "1" ((long)(a1)), "2" ((long)(a2))		\
 		: "memory" );					\
 	(type)__res;						\
@@ -81,7 +78,7 @@ extern char hypercall_page[PAGE_SIZE];
 	long __res, __ign1, __ign2, __ign3;			\
 	asm volatile (						\
 		"call hypercall_page + ("STR(__HYPERVISOR_##name)" * 32)"\
-		: "=a" (__res), "=D" (__ign1), "=S" (__ign2), 	\
+		: "=a" (__res), "=b" (__ign1), "=c" (__ign2), 	\
 		"=d" (__ign3)					\
 		: "1" ((long)(a1)), "2" ((long)(a2)),		\
 		"3" ((long)(a3))				\
@@ -91,30 +88,28 @@ extern char hypercall_page[PAGE_SIZE];
 
 #define _hypercall4(type, name, a1, a2, a3, a4)			\
 ({								\
-	long __res, __ign1, __ign2, __ign3;			\
+	long __res, __ign1, __ign2, __ign3, __ign4;		\
 	asm volatile (						\
-		"movq %7,%%r10; "				\
 		"call hypercall_page + ("STR(__HYPERVISOR_##name)" * 32)"\
-		: "=a" (__res), "=D" (__ign1), "=S" (__ign2),	\
-		"=d" (__ign3)					\
+		: "=a" (__res), "=b" (__ign1), "=c" (__ign2),	\
+		"=d" (__ign3), "=S" (__ign4)			\
 		: "1" ((long)(a1)), "2" ((long)(a2)),		\
-		"3" ((long)(a3)), "g" ((long)(a4))		\
-		: "memory", "r10" );				\
+		"3" ((long)(a3)), "4" ((long)(a4))		\
+		: "memory" );					\
 	(type)__res;						\
 })
 
 #define _hypercall5(type, name, a1, a2, a3, a4, a5)		\
 ({								\
-	long __res, __ign1, __ign2, __ign3;			\
+	long __res, __ign1, __ign2, __ign3, __ign4, __ign5;	\
 	asm volatile (						\
-		"movq %7,%%r10; movq %8,%%r8; "			\
 		"call hypercall_page + ("STR(__HYPERVISOR_##name)" * 32)"\
-		: "=a" (__res), "=D" (__ign1), "=S" (__ign2),	\
-		"=d" (__ign3)					\
+		: "=a" (__res), "=b" (__ign1), "=c" (__ign2),	\
+		"=d" (__ign3), "=S" (__ign4), "=D" (__ign5)	\
 		: "1" ((long)(a1)), "2" ((long)(a2)),		\
-		"3" ((long)(a3)), "g" ((long)(a4)),		\
-		"g" ((long)(a5))				\
-		: "memory", "r10", "r8" );			\
+		"3" ((long)(a3)), "4" ((long)(a4)),		\
+		"5" ((long)(a5))				\
+		: "memory" );					\
 	(type)__res;						\
 })
 
@@ -155,11 +150,12 @@ HYPERVISOR_stack_switch(
 
 static inline int
 HYPERVISOR_set_callbacks(
-	unsigned long event_address, unsigned long failsafe_address, 
-	unsigned long syscall_address)
+	unsigned long event_selector, unsigned long event_address,
+	unsigned long failsafe_selector, unsigned long failsafe_address)
 {
-	return _hypercall3(int, set_callbacks,
-			   event_address, failsafe_address, syscall_address);
+	return _hypercall4(int, set_callbacks,
+			   event_selector, event_address,
+			   failsafe_selector, failsafe_address);
 }
 
 static inline int
@@ -171,7 +167,7 @@ HYPERVISOR_fpu_taskswitch(
 
 static inline int
 HYPERVISOR_sched_op(
-	int cmd, unsigned long arg)
+	int cmd, void *arg)
 {
 	return _hypercall2(int, sched_op, cmd, arg);
 }
@@ -180,15 +176,9 @@ static inline long
 HYPERVISOR_set_timer_op(
 	u64 timeout)
 {
-	return _hypercall1(long, set_timer_op, timeout);
-}
-
-static inline int
-HYPERVISOR_dom0_op(
-	dom0_op_t *dom0_op)
-{
-	dom0_op->interface_version = DOM0_INTERFACE_VERSION;
-	return _hypercall1(int, dom0_op, dom0_op);
+	unsigned long timeout_hi = (unsigned long)(timeout>>32);
+	unsigned long timeout_lo = (unsigned long)timeout;
+	return _hypercall2(long, set_timer_op, timeout_lo, timeout_hi);
 }
 
 static inline int
@@ -207,9 +197,9 @@ HYPERVISOR_get_debugreg(
 
 static inline int
 HYPERVISOR_update_descriptor(
-	unsigned long ma, unsigned long word)
+	u64 ma, u64 desc)
 {
-	return _hypercall2(int, update_descriptor, ma, word);
+	return _hypercall4(int, update_descriptor, ma, ma>>32, desc, desc>>32);
 }
 
 static inline int
@@ -230,14 +220,19 @@ static inline int
 HYPERVISOR_update_va_mapping(
 	unsigned long va, pte_t new_val, unsigned long flags)
 {
-	return _hypercall3(int, update_va_mapping, va, new_val.pte, flags);
+	unsigned long pte_hi = 0;
+#ifdef CONFIG_X86_PAE
+	pte_hi = new_val.pte_high;
+#endif
+	return _hypercall4(int, update_va_mapping, va,
+			   new_val.pte_low, pte_hi, flags);
 }
 
 static inline int
 HYPERVISOR_event_channel_op(
-	void *op)
+	int cmd, void *op)
 {
-	return _hypercall1(int, event_channel_op, op);
+	return _hypercall2(int, event_channel_op, cmd, op);
 }
 
 static inline int
@@ -272,8 +267,12 @@ static inline int
 HYPERVISOR_update_va_mapping_otherdomain(
 	unsigned long va, pte_t new_val, unsigned long flags, domid_t domid)
 {
-	return _hypercall4(int, update_va_mapping_otherdomain, va,
-			   new_val.pte, flags, domid);
+	unsigned long pte_hi = 0;
+#ifdef CONFIG_X86_PAE
+	pte_hi = new_val.pte_high;
+#endif
+	return _hypercall5(int, update_va_mapping_otherdomain, va,
+			   new_val.pte_low, pte_hi, flags, domid);
 }
 
 static inline int
@@ -290,13 +289,6 @@ HYPERVISOR_vcpu_op(
 	return _hypercall3(int, vcpu_op, cmd, vcpuid, extra_args);
 }
 
-static inline int
-HYPERVISOR_set_segment_base(
-	int reg, unsigned long value)
-{
-	return _hypercall2(int, set_segment_base, reg, value);
-}
-
 static inline int
 HYPERVISOR_suspend(
 	unsigned long srec)
@@ -313,7 +305,21 @@ HYPERVISOR_nmi_op(
 	return _hypercall2(int, nmi_op, op, arg);
 }
 
-#endif /* __HYPERCALL_X86_64_H__ */
+static inline int
+HYPERVISOR_sysctl(
+	unsigned long op)
+{
+	return _hypercall1(int, sysctl, op);
+}
+
+static inline int
+HYPERVISOR_domctl(
+	unsigned long op)
+{
+	return _hypercall1(int, domctl, op);
+}
+
+#endif /* __HYPERCALL_X86_32_H__ */
 
 /*
  * Local variables:
