@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004 Mike Wray <mike.wray@hp.com>
+ * Copyright (C) 2004, 2005 Mike Wray <mike.wray@hp.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the 
@@ -51,6 +51,7 @@
 #include <tunnel.h>
 #include <vnet.h>
 #include <skb_util.h>
+#include <skb_context.h>
 
 static const int DEBUG_ICV = 0;
 
@@ -59,6 +60,18 @@ static const int DEBUG_ICV = 0;
 #undef DEBUG
 #include "debug.h"
 
+#ifndef CONFIG_CRYPTO_HMAC
+#warning No esp transform - CONFIG_CRYPTO_HMAC not defined
+
+int __init esp_module_init(void){
+    return 0;
+}
+
+void __exit esp_module_exit(void){
+}
+
+#else
+
 /* Outgoing packet:                            [ eth | ip | data ]
  * After etherip:        [ eth2 | ip2 |  ethip | eth | ip | data ]
  * After esp   :   [ eth2 | ip2 | esp | {ethip | eth | ip | data} | pad | icv ]
@@ -91,7 +104,7 @@ static const int DEBUG_ICV = 0;
  * @param block size to round to a multiple of
  * @return rounded value
  */
-static inline int roundup(int n, int block){
+static inline int roundupto(int n, int block){
     if(block <= 1) return n;
     block--;
     return (n + block) & ~block;
@@ -221,7 +234,7 @@ static int esp_sa_digest(ESPState *esp, struct sk_buff *skb, int digest_n, int i
     
     if(DEBUG_ICV){
         dprintf("> skb digest_n=%d icv_n=%d\n", digest_n, icv_n);
-        skb_print_bits(skb, 0, digest_n);
+        skb_print_bits("esp", skb, 0, digest_n);
     }
     memset(icv, 0, icv_n);
     esp->digest.icv(esp, skb, 0, digest_n, icv);
@@ -248,7 +261,7 @@ static int esp_check_icv(SAState *sa, ESPState *esp, ESPHdr *esph, struct sk_buf
     if(DEBUG_ICV){
         dprintf("> skb len=%d digest_n=%d icv_n=%d\n",
                 skb->len, digest_n, icv_n);
-        skb_print_bits(skb, 0, skb->len);
+        skb_print_bits("esp", skb, 0, skb->len);
     }
     if(skb_copy_bits(skb, digest_n, icv_skb, icv_n)){
         wprintf("> Error getting icv from skb\n");
@@ -299,9 +312,9 @@ static int esp_sa_send(SAState *sa, struct sk_buff *skb, Tunnel *tunnel){
     // header and IP header.
     plaintext_n = skb->len - ETH_HLEN - ip_n;
     // Add size of padding fields.
-    ciphertext_n = roundup(plaintext_n + ESP_PAD_N, esp->cipher.block_n);
+    ciphertext_n = roundupto(plaintext_n + ESP_PAD_N, esp->cipher.block_n);
     if(esp->cipher.pad_n > 0){
-        ciphertext_n = roundup(ciphertext_n, esp->cipher.pad_n);
+        ciphertext_n = roundupto(ciphertext_n, esp->cipher.pad_n);
     }
     extra_n = ciphertext_n - plaintext_n;
     iv_n = esp->cipher.iv_n;
@@ -309,7 +322,7 @@ static int esp_sa_send(SAState *sa, struct sk_buff *skb, Tunnel *tunnel){
     dprintf("> len=%d plaintext=%d ciphertext=%d extra=%d\n",
             skb->len, plaintext_n, ciphertext_n, extra_n);
     dprintf("> iv=%d icv=%d\n", iv_n, icv_n);
-    skb_print_bits(skb, 0, skb->len);
+    skb_print_bits("iv", skb, 0, skb->len);
 
     // Add headroom for esp header and iv, tailroom for the ciphertext
     // and icv.
@@ -328,12 +341,12 @@ static int esp_sa_send(SAState *sa, struct sk_buff *skb, Tunnel *tunnel){
         dprintf("> ETH header pull...\n");
         memmove(skb->data, skb->mac.raw, ETH_HLEN);
         skb->mac.raw = skb->data; 
-        __skb_pull(skb, ETH_HLEN);
+        skb_pull_vn(skb, ETH_HLEN);
     }
     dprintf("> IP header pull...\n");
     memmove(skb->data, skb->nh.raw, ip_n);
     skb->nh.raw = skb->data;
-    __skb_pull(skb, ip_n);
+    skb_pull_vn(skb, ip_n);
     esph = (void*)skb->data;
     // Add spi and sequence number.
     esph->spi = sa->ident.spi;
@@ -393,9 +406,12 @@ static void esp_context_free_fn(SkbContext *context){
  * Does ESP receive processing (check icv, decrypt), strips
  * ESP header and re-receives.
  *
+ * If return 1 the packet has been freed.
+ * If return <= 0 the caller must free.
+ *
  * @param sa SA
  * @param skb packet
- * @return 0 on success, negative error code otherwise
+ * @return >= 0 on success, negative protocol otherwise
  */
 static int esp_sa_recv(SAState *sa, struct sk_buff *skb){
     int err = -EINVAL;
@@ -441,7 +457,7 @@ static int esp_sa_recv(SAState *sa, struct sk_buff *skb){
     // Move skb->data back to ethernet header.
     // Do in 2 moves to ensure offsets are +ve,
     // since args to skb_pull/skb_push are unsigned.
-    __skb_pull(skb, head_n);
+    skb_pull_vn(skb, head_n);
     __skb_push(skb, skb->data - skb->mac.raw);
     // After this esph is invalid.
     esph = NULL;
@@ -458,10 +474,19 @@ static int esp_sa_recv(SAState *sa, struct sk_buff *skb){
                            sa, esp_context_free_fn);
     if(err) goto exit;
     // Increase sa refcount now the skb context refers to it.
+    // Refcount is decreased by esp_context_free_fn.
     SAState_incref(sa);
-    err = netif_rx(skb);
+    // Deliver skb to be received by network code.
+    // Not safe to refer to the skb after this.
+    // todo: return -skb->nh.iph->protocol instead?
+    netif_rx(skb);
   exit:
-    if(mine) err = 1;
+    if(mine){
+        if(err < 0){
+            kfree_skb(skb);
+        }
+        err = 1;
+    }
     dprintf("< skb=%p err=%d\n", skb, err);
     return err;
 }
@@ -477,9 +502,9 @@ static u32 esp_sa_size(SAState *sa, int data_n){
     // Have to add some padding for alignment even if pad_n is zero.
     ESPState *esp = sa->data;
     
-    data_n = roundup(data_n + ESP_PAD_N, esp->cipher.block_n);
+    data_n = roundupto(data_n + ESP_PAD_N, esp->cipher.block_n);
     if(esp->cipher.pad_n > 0){
-        data_n = roundup(data_n, esp->cipher.pad_n);
+        data_n = roundupto(data_n, esp->cipher.pad_n);
     }
     data_n += esp->digest.icv_n;
     //data_n += esp->cipher.iv_n;
@@ -582,7 +607,7 @@ static int esp_cipher_init(SAState *sa, ESPState *esp){
         err = -EINVAL;
         goto exit;
     }
-    esp->cipher.key_n = roundup(sa->cipher.bits, 8);
+    esp->cipher.key_n = roundupto(sa->cipher.bits, 8);
     // If cipher is null must use ECB because CBC algo does not support blocksize 1.
     if(strcmp(sa->cipher.name, "cipher_null")){
         cipher_mode = CRYPTO_TFM_MODE_ECB;
@@ -592,7 +617,7 @@ static int esp_cipher_init(SAState *sa, ESPState *esp){
         err = -ENOMEM;
         goto exit;
     }
-    esp->cipher.block_n = roundup(crypto_tfm_alg_blocksize(esp->cipher.tfm), 4);
+    esp->cipher.block_n = roundupto(crypto_tfm_alg_blocksize(esp->cipher.tfm), 4);
     esp->cipher.iv_n = crypto_tfm_alg_ivsize(esp->cipher.tfm);
     esp->cipher.pad_n = 0;
     if(esp->cipher.iv_n){
@@ -618,7 +643,7 @@ static int esp_digest_init(SAState *sa, ESPState *esp){
     
     dprintf(">\n");
     esp->digest.key = sa->digest.key;
-    esp->digest.key_n = bits_to_bytes(roundup(sa->digest.bits, 8));
+    esp->digest.key_n = bits_to_bytes(roundupto(sa->digest.bits, 8));
     esp->digest.tfm = crypto_alloc_tfm(sa->digest.name, 0);
     if(!esp->digest.tfm){
         err = -ENOMEM;
@@ -717,9 +742,15 @@ static int esp_skb_header(struct sk_buff *skb, ESPHdr **esph){
  * Lookup spi, if state found hand to the state.
  * If no state, check spi, if ok, create state and pass to it.
  * If spi not ok, drop.
+ *
+ * Return value convention for protocols:
+ * >= 0 Protocol took the packet
+ * < 0  A -ve protocol id the packet should be re-received as.
+ *
+ * So always return >=0 if we took the packet, even if we dropped it.
  * 
  * @param skb packet
- * @return 0 on sucess, negative error code otherwise
+ * @return 0 on sucess, negative protocol number otherwise
  */
 static int esp_protocol_recv(struct sk_buff *skb){
     int err = 0;
@@ -730,7 +761,10 @@ static int esp_protocol_recv(struct sk_buff *skb){
     u32 addr;
     
     dprintf(">\n");
-    dprintf("> recv skb=\n"); skb_print_bits(skb, 0, skb->len);
+#ifdef DEBUG
+    dprintf("> recv skb=\n"); 
+    skb_print_bits("", skb, 0, skb->len);
+#endif
     ip_n = (skb->nh.iph->ihl << 2);
     if(skb->data == skb->mac.raw){
         // skb->data points at ethernet header.
@@ -739,7 +773,7 @@ static int esp_protocol_recv(struct sk_buff *skb){
             err = -EINVAL;
             goto exit;
         }
-        skb_pull(skb, eth_n + ip_n);
+        skb_pull_vn(skb, eth_n + ip_n);
     }
     addr = skb->nh.iph->daddr;
     err = esp_skb_header(skb, &esph);
@@ -751,9 +785,14 @@ static int esp_protocol_recv(struct sk_buff *skb){
         err = vnet_sa_create(esph->spi, IPPROTO_ESP, addr, &sa);
         if(err) goto exit;
     }
+    //todo: Return a -ve protocol instead? See esp_sa_recv.
     err = SAState_recv(sa, skb);
   exit:
     if(sa) SAState_decref(sa);
+    if(err <= 0){
+        kfree_skb(skb);
+        err = 0;
+    }
     dprintf("< err=%d\n", err);
     return err;
 }
@@ -861,3 +900,4 @@ void __exit esp_module_exit(void){
     }
 }
 
+#endif // CONFIG_CRYPTO_HMAC
