@@ -23,13 +23,12 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-//#define DEBUG
 #include "utils.h"
 #include "talloc.h"
 #include "xenstored_core.h"
 #include "xenstored_domain.h"
+#include "xenstored_transaction.h"
 #include "xenstored_watch.h"
-#include "xenstored_test.h"
 
 #include <xenctrl.h>
 
@@ -77,7 +76,6 @@ struct domain
 
 static LIST_HEAD(domains);
 
-/* FIXME: Mark connection as broken (close it?) when this happens. */
 static bool check_indexes(XENSTORE_RING_IDX cons, XENSTORE_RING_IDX prod)
 {
 	return ((prod - cons) <= XENSTORE_RING_SIZE);
@@ -103,7 +101,8 @@ static const void *get_input_chunk(XENSTORE_RING_IDX cons,
 	return buf + MASK_XENSTORE_IDX(cons);
 }
 
-static int writechn(struct connection *conn, const void *data, unsigned int len)
+static int writechn(struct connection *conn,
+		    const void *data, unsigned int len)
 {
 	uint32_t avail;
 	void *dest;
@@ -114,6 +113,7 @@ static int writechn(struct connection *conn, const void *data, unsigned int len)
 	cons = intf->rsp_cons;
 	prod = intf->rsp_prod;
 	mb();
+
 	if (!check_indexes(cons, prod)) {
 		errno = EIO;
 		return -1;
@@ -176,6 +176,8 @@ static int destroy_domain(void *_domain)
 	if (domain->interface)
 		munmap(domain->interface, getpagesize());
 
+	fire_watches(NULL, "@releaseDomain", false);
+
 	return 0;
 }
 
@@ -198,7 +200,7 @@ static void domain_cleanup(void)
 				continue;
 		}
 		talloc_free(domain->conn);
-		notify = 1;
+		notify = 0; /* destroy_domain() fires the watch */
 	}
 
 	if (notify)
@@ -216,10 +218,8 @@ void handle_event(void)
 	if (port == virq_port)
 		domain_cleanup();
 
-#ifndef TESTING
 	if (xc_evtchn_unmask(xce_handle, port) == -1)
 		barf_perror("Failed to write to event fd");
-#endif
 }
 
 bool domain_can_read(struct connection *conn)
@@ -250,7 +250,6 @@ static struct domain *new_domain(void *context, unsigned int domid,
 	struct domain *domain;
 	int rc;
 
-
 	domain = talloc(context, struct domain);
 	domain->port = 0;
 	domain->shutdown = 0;
@@ -289,6 +288,26 @@ static struct domain *find_domain_by_domid(unsigned int domid)
 	return NULL;
 }
 
+static void domain_conn_reset(struct domain *domain)
+{
+	struct connection *conn = domain->conn;
+	struct buffered_data *out;
+
+	conn_delete_all_watches(conn);
+	conn_delete_all_transactions(conn);
+
+	while ((out = list_top(&conn->out_list, struct buffered_data, list))) {
+		list_del(&out->list);
+		talloc_free(out);
+	}
+
+	talloc_free(conn->in->buffer);
+	memset(conn->in, 0, sizeof(*conn->in));
+	conn->in->inhdr = true;
+
+	domain->interface->req_cons = domain->interface->req_prod = 0;
+	domain->interface->rsp_cons = domain->interface->rsp_prod = 0;
+}
 
 /* domid, mfn, evtchn, path */
 void do_introduce(struct connection *conn, struct buffered_data *in)
@@ -298,6 +317,8 @@ void do_introduce(struct connection *conn, struct buffered_data *in)
 	unsigned int domid;
 	unsigned long mfn;
 	evtchn_port_t port;
+	int rc;
+	struct xenstore_domain_interface *interface;
 
 	if (get_strings(in, vec, ARRAY_SIZE(vec)) < ARRAY_SIZE(vec)) {
 		send_error(conn, EINVAL);
@@ -322,36 +343,41 @@ void do_introduce(struct connection *conn, struct buffered_data *in)
 	domain = find_domain_by_domid(domid);
 
 	if (domain == NULL) {
+		interface = xc_map_foreign_range(
+			*xc_handle, domid,
+			getpagesize(), PROT_READ|PROT_WRITE, mfn);
+		if (!interface) {
+			send_error(conn, errno);
+			return;
+		}
 		/* Hang domain off "in" until we're finished. */
 		domain = new_domain(in, domid, port);
 		if (!domain) {
+			munmap(interface, getpagesize());
 			send_error(conn, errno);
 			return;
 		}
-		domain->interface = xc_map_foreign_range(
-			*xc_handle, domid,
-			getpagesize(), PROT_READ|PROT_WRITE, mfn);
-		if (!domain->interface) {
-			send_error(conn, errno);
-			return;
-		}
+		domain->interface = interface;
 		domain->mfn = mfn;
 
 		/* Now domain belongs to its connection. */
 		talloc_steal(domain->conn, domain);
 
-		fire_watches(conn, "@introduceDomain", false);
-	}
-	else {
-		/* Check that the given details match the ones we have
-		   previously recorded. */
-		if (port != domain->remote_port ||
-		    mfn != domain->mfn) {
-			send_error(conn, EINVAL);
-			return;
-		}
+		fire_watches(NULL, "@introduceDomain", false);
+	} else if ((domain->mfn == mfn) && (domain->conn != conn)) {
+		/* Use XS_INTRODUCE for recreating the xenbus event-channel. */
+		if (domain->port)
+			xc_evtchn_unbind(xce_handle, domain->port);
+		rc = xc_evtchn_bind_interdomain(xce_handle, domid, port);
+		domain->port = (rc == -1) ? 0 : rc;
+		domain->remote_port = port;
+	} else {
+		send_error(conn, EINVAL);
+		return;
 	}
 
+	domain_conn_reset(domain);
+
 	send_ack(conn, XS_INTRODUCE);
 }
 
@@ -390,11 +416,46 @@ void do_release(struct connection *conn, const char *domid_str)
 
 	talloc_free(domain->conn);
 
-	fire_watches(conn, "@releaseDomain", false);
-
 	send_ack(conn, XS_RELEASE);
 }
 
+void do_resume(struct connection *conn, const char *domid_str)
+{
+	struct domain *domain;
+	unsigned int domid;
+
+	if (!domid_str) {
+		send_error(conn, EINVAL);
+		return;
+	}
+
+	domid = atoi(domid_str);
+	if (!domid) {
+		send_error(conn, EINVAL);
+		return;
+	}
+
+	if (conn->id != 0) {
+		send_error(conn, EACCES);
+		return;
+	}
+
+	domain = find_domain_by_domid(domid);
+	if (!domain) {
+		send_error(conn, ENOENT);
+		return;
+	}
+
+	if (!domain->conn) {
+		send_error(conn, EINVAL);
+		return;
+	}
+
+	domain->shutdown = 0;
+	
+	send_ack(conn, XS_RESUME);
+}
+
 void do_get_domain_path(struct connection *conn, const char *domid_str)
 {
 	char *path;
@@ -459,6 +520,8 @@ static int dom0_init(void)
 		return -1;
 
 	dom0 = new_domain(NULL, 0, port); 
+	if (dom0 == NULL)
+		return -1;
 
 	dom0->interface = xenbus_map();
 	if (dom0->interface == NULL)
@@ -501,19 +564,65 @@ int domain_init(void)
 	return xce_handle;
 }
 
-void domain_entry_inc(struct connection *conn)
+void domain_entry_inc(struct connection *conn, struct node *node)
 {
-	if (!conn || !conn->domain)
+	struct domain *d;
+
+	if (!conn)
 		return;
-	conn->domain->nbentry++;
+
+	if (node->perms && node->perms[0].id != conn->id) {
+		if (conn->transaction) {
+			transaction_entry_inc(conn->transaction,
+				node->perms[0].id);
+		} else {
+			d = find_domain_by_domid(node->perms[0].id);
+			if (d)
+				d->nbentry++;
+		}
+	} else if (conn->domain) {
+		if (conn->transaction) {
+			transaction_entry_inc(conn->transaction,
+				conn->domain->domid);
+ 		} else {
+ 			conn->domain->nbentry++;
+		}
+	}
 }
 
-void domain_entry_dec(struct connection *conn)
+void domain_entry_dec(struct connection *conn, struct node *node)
 {
-	if (!conn || !conn->domain)
+	struct domain *d;
+
+	if (!conn)
 		return;
-	if (conn->domain->nbentry)
-		conn->domain->nbentry--;
+
+	if (node->perms && node->perms[0].id != conn->id) {
+		if (conn->transaction) {
+			transaction_entry_dec(conn->transaction,
+				node->perms[0].id);
+		} else {
+			d = find_domain_by_domid(node->perms[0].id);
+			if (d && d->nbentry)
+				d->nbentry--;
+		}
+	} else if (conn->domain && conn->domain->nbentry) {
+		if (conn->transaction) {
+			transaction_entry_dec(conn->transaction,
+				conn->domain->domid);
+		} else {
+			conn->domain->nbentry--;
+		}
+	}
+}
+
+void domain_entry_fix(unsigned int domid, int num)
+{
+	struct domain *d;
+
+	d = find_domain_by_domid(domid);
+	if (d && ((d->nbentry += num) < 0))
+		d->nbentry = 0;
 }
 
 int domain_entry(struct connection *conn)
