@@ -30,9 +30,11 @@
 #include <asm/vmx.h>
 #include <asm/virt_event.h>
 #include <asm/vmx_phy_mode.h>
+#include <asm/debugger.h>
 
-void
-ia64_priv_decoder(IA64_SLOT_TYPE slot_type, INST64 inst, UINT64  * cause)
+#ifdef BYPASS_VMAL_OPCODE
+static void
+ia64_priv_decoder(IA64_SLOT_TYPE slot_type, INST64 inst, u64 * cause)
 {
     *cause=0;
     switch (slot_type) {
@@ -141,23 +143,24 @@ ia64_priv_decoder(IA64_SLOT_TYPE slot_type, INST64 inst, UINT64  * cause)
         break;
     }
 }
+#endif
 
-IA64FAULT vmx_emul_rsm(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_rsm(VCPU *vcpu, INST64 inst)
 {
-    UINT64 imm24 = (inst.M44.i<<23)|(inst.M44.i2<<21)|inst.M44.imm;
+    u64 imm24 = (inst.M44.i << 23) | (inst.M44.i2 << 21) | inst.M44.imm;
     return vmx_vcpu_reset_psr_sm(vcpu,imm24);
 }
 
-IA64FAULT vmx_emul_ssm(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ssm(VCPU *vcpu, INST64 inst)
 {
-    UINT64 imm24 = (inst.M44.i<<23)|(inst.M44.i2<<21)|inst.M44.imm;
+    u64 imm24 = (inst.M44.i << 23) | (inst.M44.i2 << 21) | inst.M44.imm;
     return vmx_vcpu_set_psr_sm(vcpu,imm24);
 }
 
-IA64FAULT vmx_emul_mov_from_psr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_from_psr(VCPU *vcpu, INST64 inst)
 {
-    UINT64 tgt = inst.M33.r1;
-    UINT64 val;
+    u64 tgt = inst.M33.r1;
+    u64 val;
 
 /*
     if ((fault = vmx_vcpu_get_psr(vcpu,&val)) == IA64_NO_FAULT)
@@ -172,12 +175,12 @@ IA64FAULT vmx_emul_mov_from_psr(VCPU *vcpu, INST64 inst)
 /**
  * @todo Check for reserved bits and return IA64_RSVDREG_FAULT.
  */
-IA64FAULT vmx_emul_mov_to_psr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_psr(VCPU *vcpu, INST64 inst)
 {
-    UINT64 val;
+    u64 val;
 
-    if(vcpu_get_gr_nat(vcpu, inst.M35.r2, &val) != IA64_NO_FAULT)
-	panic_domain(vcpu_regs(vcpu),"get_psr nat bit fault\n");
+    if (vcpu_get_gr_nat(vcpu, inst.M35.r2, &val) != IA64_NO_FAULT)
+        panic_domain(vcpu_regs(vcpu),"get_psr nat bit fault\n");
 
     return vmx_vcpu_set_psr_l(vcpu, val);
 }
@@ -187,7 +190,7 @@ IA64FAULT vmx_emul_mov_to_psr(VCPU *vcpu, INST64 inst)
 Privileged operation emulation routines
 **************************************************************************/
 
-IA64FAULT vmx_emul_rfi(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_rfi(VCPU *vcpu, INST64 inst)
 {
     IA64_PSR  vpsr;
     REGS *regs;
@@ -200,6 +203,12 @@ IA64FAULT vmx_emul_rfi(VCPU *vcpu, INST64 inst)
         return IA64_FAULT;
     }
 #endif // CHECK_FAULT
+
+    if (debugger_event(XEN_IA64_DEBUG_ON_RFI)) {
+        raise_softirq(SCHEDULE_SOFTIRQ);
+        do_softirq();
+    }
+
     regs=vcpu_regs(vcpu);
     vpsr.val=regs->cr_ipsr;
     if ( vpsr.is == 1 ) {
@@ -209,7 +218,7 @@ IA64FAULT vmx_emul_rfi(VCPU *vcpu, INST64 inst)
     return vmx_vcpu_rfi(vcpu);
 }
 
-IA64FAULT vmx_emul_bsw0(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_bsw0(VCPU *vcpu, INST64 inst)
 {
 #ifdef  CHECK_FAULT
     IA64_PSR  vpsr;
@@ -224,7 +233,7 @@ IA64FAULT vmx_emul_bsw0(VCPU *vcpu, INST64 inst)
    return vcpu_bsw0(vcpu);
 }
 
-IA64FAULT vmx_emul_bsw1(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_bsw1(VCPU *vcpu, INST64 inst)
 {
 #ifdef  CHECK_FAULT
     IA64_PSR  vpsr;
@@ -239,12 +248,12 @@ IA64FAULT vmx_emul_bsw1(VCPU *vcpu, INST64 inst)
     return vcpu_bsw1(vcpu);
 }
 
-IA64FAULT vmx_emul_cover(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_cover(VCPU *vcpu, INST64 inst)
 {
     return vmx_vcpu_cover(vcpu);
 }
 
-IA64FAULT vmx_emul_ptc_l(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ptc_l(VCPU *vcpu, INST64 inst)
 {
     u64 r2,r3;
 #ifdef  VMAL_NO_FAULT_CHECK
@@ -273,12 +282,15 @@ IA64FAULT vmx_emul_ptc_l(VCPU *vcpu, INST64 inst)
         vcpu_set_isr(vcpu, isr.val);
         unimpl_daddr(vcpu);
         return IA64_FAULT;
-   }
+    }
 #endif // VMAL_NO_FAULT_CHECK
+
+    debugger_event(XEN_IA64_DEBUG_ON_TC);
+
     return vmx_vcpu_ptc_l(vcpu,r3,bits(r2,2,7));
 }
 
-IA64FAULT vmx_emul_ptc_e(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ptc_e(VCPU *vcpu, INST64 inst)
 {
     u64 r3;
 #ifdef  VMAL_NO_FAULT_CHECK
@@ -303,7 +315,7 @@ IA64FAULT vmx_emul_ptc_e(VCPU *vcpu, INST64 inst)
     return vmx_vcpu_ptc_e(vcpu,r3);
 }
 
-IA64FAULT vmx_emul_ptc_g(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ptc_g(VCPU *vcpu, INST64 inst)
 {
     u64 r2,r3;
 #ifdef  VMAL_NO_FAULT_CHECK    
@@ -331,12 +343,15 @@ IA64FAULT vmx_emul_ptc_g(VCPU *vcpu, INST64 inst)
         vcpu_set_isr(vcpu, isr.val);
         unimpl_daddr(vcpu);
         return IA64_FAULT;
-   }
+    }
 #endif // VMAL_NO_FAULT_CHECK
+
+    debugger_event(XEN_IA64_DEBUG_ON_TC);
+
     return vmx_vcpu_ptc_g(vcpu,r3,bits(r2,2,7));
 }
 
-IA64FAULT vmx_emul_ptc_ga(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ptc_ga(VCPU *vcpu, INST64 inst)
 {
     u64 r2,r3;
 #ifdef  VMAL_NO_FAULT_CHECK    
@@ -364,12 +379,15 @@ IA64FAULT vmx_emul_ptc_ga(VCPU *vcpu, INST64 inst)
         vcpu_set_isr(vcpu, isr.val);
         unimpl_daddr(vcpu);
         return IA64_FAULT;
-   }
+    }
 #endif // VMAL_NO_FAULT_CHECK
+
+    debugger_event(XEN_IA64_DEBUG_ON_TC);
+
     return vmx_vcpu_ptc_ga(vcpu,r3,bits(r2,2,7));
 }
 
-IA64FAULT ptr_fault_check(VCPU *vcpu, INST64 inst, u64 *pr2, u64 *pr3)
+static IA64FAULT ptr_fault_check(VCPU *vcpu, INST64 inst, u64 *pr2, u64 *pr3)
 {
     IA64FAULT	ret1, ret2;
 
@@ -403,24 +421,30 @@ IA64FAULT ptr_fault_check(VCPU *vcpu, INST64 inst, u64 *pr2, u64 *pr3)
    return IA64_NO_FAULT;
 }
 
-IA64FAULT vmx_emul_ptr_d(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ptr_d(VCPU *vcpu, INST64 inst)
 {
     u64 r2,r3;
     if ( ptr_fault_check(vcpu, inst, &r2, &r3 ) == IA64_FAULT )
     	return IA64_FAULT;
+
+    debugger_event(XEN_IA64_DEBUG_ON_TR);
+
     return vmx_vcpu_ptr_d(vcpu,r3,bits(r2,2,7));
 }
 
-IA64FAULT vmx_emul_ptr_i(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ptr_i(VCPU *vcpu, INST64 inst)
 {
     u64 r2,r3;
     if ( ptr_fault_check(vcpu, inst, &r2, &r3 ) == IA64_FAULT )
     	return IA64_FAULT;
+
+    debugger_event(XEN_IA64_DEBUG_ON_TR);
+
     return vmx_vcpu_ptr_i(vcpu,r3,bits(r2,2,7));
 }
 
 
-IA64FAULT vmx_emul_thash(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_thash(VCPU *vcpu, INST64 inst)
 {
     u64 r1,r3;
 #ifdef  CHECK_FAULT
@@ -444,13 +468,13 @@ IA64FAULT vmx_emul_thash(VCPU *vcpu, INST64 inst)
         return IA64_NO_FAULT;
     }
 #endif  //CHECK_FAULT
-    vmx_vcpu_thash(vcpu, r3, &r1);
+    r1 = vmx_vcpu_thash(vcpu, r3);
     vcpu_set_gr(vcpu, inst.M46.r1, r1, 0);
     return(IA64_NO_FAULT);
 }
 
 
-IA64FAULT vmx_emul_ttag(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_ttag(VCPU *vcpu, INST64 inst)
 {
     u64 r1,r3;
 #ifdef  CHECK_FAULT
@@ -476,13 +500,13 @@ IA64FAULT vmx_emul_ttag(VCPU *vcpu, INST64 inst)
         return IA64_NO_FAULT;
     }
 #endif  //CHECK_FAULT
-    vmx_vcpu_ttag(vcpu, r3, &r1);
+    r1 = vmx_vcpu_ttag(vcpu, r3);
     vcpu_set_gr(vcpu, inst.M46.r1, r1, 0);
     return(IA64_NO_FAULT);
 }
 
 
-IA64FAULT vmx_emul_tpa(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_tpa(VCPU *vcpu, INST64 inst)
 {
     u64 r1,r3;
 #ifdef  CHECK_FAULT
@@ -526,7 +550,7 @@ IA64FAULT vmx_emul_tpa(VCPU *vcpu, INST64 inst)
     return(IA64_NO_FAULT);
 }
 
-IA64FAULT vmx_emul_tak(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_tak(VCPU *vcpu, INST64 inst)
 {
     u64 r1,r3;
 #ifdef  CHECK_FAULT
@@ -552,9 +576,7 @@ IA64FAULT vmx_emul_tak(VCPU *vcpu, INST64 inst)
         return IA64_FAULT;
 #endif
     }
-    if(vmx_vcpu_tak(vcpu, r3, &r1)){
-        return IA64_FAULT;
-    }
+    r1 = vmx_vcpu_tak(vcpu, r3);
     vcpu_set_gr(vcpu, inst.M46.r1, r1, 0);
     return(IA64_NO_FAULT);
 }
@@ -564,44 +586,47 @@ IA64FAULT vmx_emul_tak(VCPU *vcpu, INST64 inst)
  * Insert translation register/cache
 ************************************/
 
-IA64FAULT vmx_emul_itr_d(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_itr_d(VCPU *vcpu, INST64 inst)
 {
-    UINT64 itir, ifa, pte, slot;
+    u64 itir, ifa, pte, slot;
+    ISR isr;
+
 #ifdef  VMAL_NO_FAULT_CHECK
     IA64_PSR  vpsr;
-    vpsr.val=vmx_vcpu_get_psr(vcpu);
-    if ( vpsr.ic ) {
+
+    vpsr.val = vmx_vcpu_get_psr(vcpu);
+    if (vpsr.ic) {
         set_illegal_op_isr(vcpu);
         illegal_op(vcpu);
         return IA64_FAULT;
     }
-    ISR isr;
-    if ( vpsr.cpl != 0) {
+    if (vpsr.cpl != 0) {
         /* Inject Privileged Operation fault into guest */
-        set_privileged_operation_isr (vcpu, 0);
+        set_privileged_operation_isr(vcpu, 0);
         privilege_op (vcpu);
         return IA64_FAULT;
     }
 #endif // VMAL_NO_FAULT_CHECK
-    if(vcpu_get_gr_nat(vcpu,inst.M45.r3,&slot)||vcpu_get_gr_nat(vcpu,inst.M45.r2,&pte)){
+    if (vcpu_get_gr_nat(vcpu, inst.M45.r3, &slot)
+        || vcpu_get_gr_nat(vcpu, inst.M45.r2, &pte)) {
 #ifdef  VMAL_NO_FAULT_CHECK
-        set_isr_reg_nat_consumption(vcpu,0,0);
+        set_isr_reg_nat_consumption(vcpu, 0, 0);
         rnat_comsumption(vcpu);
         return IA64_FAULT;
 #endif // VMAL_NO_FAULT_CHECK
     }
 #ifdef  VMAL_NO_FAULT_CHECK
-    if(is_reserved_rr_register(vcpu, slot)){
+    if (is_reserved_rr_register(vcpu, slot)) {
         set_illegal_op_isr(vcpu);
         illegal_op(vcpu);
         return IA64_FAULT;
     }
 #endif // VMAL_NO_FAULT_CHECK
 
-    if (vcpu_get_itir(vcpu,&itir)){
+    if (vcpu_get_itir(vcpu ,&itir)) {
         return(IA64_FAULT);
     }
-    if (vcpu_get_ifa(vcpu,&ifa)){
+    if (vcpu_get_ifa(vcpu, &ifa)) {
         return(IA64_FAULT);
     }
 #ifdef  VMAL_NO_FAULT_CHECK
@@ -609,7 +634,7 @@ IA64FAULT vmx_emul_itr_d(VCPU *vcpu, INST64 inst)
     	// TODO
     	return IA64_FAULT;
     }
-    if (unimplemented_gva(vcpu,ifa) ) {
+    if (unimplemented_gva(vcpu, ifa)) {
         isr.val = set_isr_ei_ni(vcpu);
         isr.code = IA64_RESERVED_REG_FAULT;
         vcpu_set_isr(vcpu, isr.val);
@@ -618,102 +643,124 @@ IA64FAULT vmx_emul_itr_d(VCPU *vcpu, INST64 inst)
    }
 #endif // VMAL_NO_FAULT_CHECK
 
-    return (vmx_vcpu_itr_d(vcpu,slot,pte,itir,ifa));
+    if (slot >= NDTRS) {
+        isr.val = set_isr_ei_ni(vcpu);
+        isr.code = IA64_RESERVED_REG_FAULT;
+        vcpu_set_isr(vcpu, isr.val);
+        rsv_reg_field(vcpu);
+        return IA64_FAULT;
+    }
+
+    debugger_event(XEN_IA64_DEBUG_ON_TR);
+
+    return (vmx_vcpu_itr_d(vcpu, slot, pte, itir, ifa));
 }
 
-IA64FAULT vmx_emul_itr_i(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_itr_i(VCPU *vcpu, INST64 inst)
 {
-    UINT64 itir, ifa, pte, slot;
-#ifdef  VMAL_NO_FAULT_CHECK
+    u64 itir, ifa, pte, slot;
     ISR isr;
+#ifdef  VMAL_NO_FAULT_CHECK
     IA64_PSR  vpsr;
-    vpsr.val=vmx_vcpu_get_psr(vcpu);
-    if ( vpsr.ic ) {
+    vpsr.val = vmx_vcpu_get_psr(vcpu);
+    if (vpsr.ic) {
         set_illegal_op_isr(vcpu);
         illegal_op(vcpu);
         return IA64_FAULT;
     }
-    if ( vpsr.cpl != 0) {
+    if (vpsr.cpl != 0) {
         /* Inject Privileged Operation fault into guest */
-        set_privileged_operation_isr (vcpu, 0);
-        privilege_op (vcpu);
+        set_privileged_operation_isr(vcpu, 0);
+        privilege_op(vcpu);
         return IA64_FAULT;
     }
 #endif // VMAL_NO_FAULT_CHECK
-    if(vcpu_get_gr_nat(vcpu,inst.M45.r3,&slot)||vcpu_get_gr_nat(vcpu,inst.M45.r2,&pte)){
+    if (vcpu_get_gr_nat(vcpu, inst.M45.r3, &slot)
+        || vcpu_get_gr_nat(vcpu, inst.M45.r2, &pte)) {
 #ifdef  VMAL_NO_FAULT_CHECK
-        set_isr_reg_nat_consumption(vcpu,0,0);
+        set_isr_reg_nat_consumption(vcpu, 0, 0);
         rnat_comsumption(vcpu);
         return IA64_FAULT;
 #endif // VMAL_NO_FAULT_CHECK
     }
 #ifdef  VMAL_NO_FAULT_CHECK
-    if(is_reserved_rr_register(vcpu, slot)){
+    if (is_reserved_rr_register(vcpu, slot)) {
         set_illegal_op_isr(vcpu);
         illegal_op(vcpu);
         return IA64_FAULT;
     }
 #endif // VMAL_NO_FAULT_CHECK
 
-    if (vcpu_get_itir(vcpu,&itir)){
-        return(IA64_FAULT);
+    if (vcpu_get_itir(vcpu, &itir)) {
+        return IA64_FAULT;
     }
-    if (vcpu_get_ifa(vcpu,&ifa)){
-        return(IA64_FAULT);
+    if (vcpu_get_ifa(vcpu, &ifa)) {
+        return IA64_FAULT;
     }
 #ifdef  VMAL_NO_FAULT_CHECK
     if (is_reserved_itir_field(vcpu, itir)) {
     	// TODO
     	return IA64_FAULT;
     }
-    if (unimplemented_gva(vcpu,ifa) ) {
+    if (unimplemented_gva(vcpu, ifa)) {
         isr.val = set_isr_ei_ni(vcpu);
         isr.code = IA64_RESERVED_REG_FAULT;
         vcpu_set_isr(vcpu, isr.val);
         unimpl_daddr(vcpu);
         return IA64_FAULT;
-   }
+    }
 #endif // VMAL_NO_FAULT_CHECK
 
-   return (vmx_vcpu_itr_i(vcpu,slot,pte,itir,ifa));
+    if (slot >= NITRS) {
+        isr.val = set_isr_ei_ni(vcpu);
+        isr.code = IA64_RESERVED_REG_FAULT;
+        vcpu_set_isr(vcpu, isr.val);
+        rsv_reg_field(vcpu);
+        return IA64_FAULT;
+    }
+
+    debugger_event(XEN_IA64_DEBUG_ON_TR);
+
+    return vmx_vcpu_itr_i(vcpu, slot, pte, itir, ifa);
 }
 
-IA64FAULT itc_fault_check(VCPU *vcpu, INST64 inst, u64 *itir, u64 *ifa,u64 *pte)
+static IA64FAULT itc_fault_check(VCPU *vcpu, INST64 inst,
+                                 u64 *itir, u64 *ifa, u64 *pte)
 {
     IA64FAULT	ret1;
 
 #ifdef  VMAL_NO_FAULT_CHECK
     IA64_PSR  vpsr;
-    vpsr.val=vmx_vcpu_get_psr(vcpu);
-    if ( vpsr.ic ) {
+    vpsr.val = vmx_vcpu_get_psr(vcpu);
+    if (vpsr.ic) {
         set_illegal_op_isr(vcpu);
         illegal_op(vcpu);
         return IA64_FAULT;
     }
 
-    UINT64 fault;
+    u64 fault;
     ISR isr;
-    if ( vpsr.cpl != 0) {
+    if (vpsr.cpl != 0) {
         /* Inject Privileged Operation fault into guest */
-        set_privileged_operation_isr (vcpu, 0);
-        privilege_op (vcpu);
+        set_privileged_operation_isr(vcpu, 0);
+        privilege_op(vcpu);
         return IA64_FAULT;
     }
 #endif // VMAL_NO_FAULT_CHECK
-    ret1 = vcpu_get_gr_nat(vcpu,inst.M45.r2,pte);
+    ret1 = vcpu_get_gr_nat(vcpu, inst.M45.r2,pte);
 #ifdef  VMAL_NO_FAULT_CHECK
-    if( ret1 != IA64_NO_FAULT ){
-        set_isr_reg_nat_consumption(vcpu,0,0);
+    if (ret1 != IA64_NO_FAULT) {
+        set_isr_reg_nat_consumption(vcpu, 0, 0);
         rnat_comsumption(vcpu);
         return IA64_FAULT;
     }
 #endif // VMAL_NO_FAULT_CHECK
 
-    if (vcpu_get_itir(vcpu,itir)){
-        return(IA64_FAULT);
+    if (vcpu_get_itir(vcpu, itir)) {
+        return IA64_FAULT;
     }
-    if (vcpu_get_ifa(vcpu,ifa)){
-        return(IA64_FAULT);
+    if (vcpu_get_ifa(vcpu, ifa)) {
+        return IA64_FAULT;
     }
 #ifdef  VMAL_NO_FAULT_CHECK
     if (unimplemented_gva(vcpu,ifa) ) {
@@ -722,42 +769,45 @@ IA64FAULT itc_fault_check(VCPU *vcpu, INST64 inst, u64 *itir, u64 *ifa,u64 *pte)
         vcpu_set_isr(vcpu, isr.val);
         unimpl_daddr(vcpu);
         return IA64_FAULT;
-   }
+    }
 #endif // VMAL_NO_FAULT_CHECK
-   return IA64_NO_FAULT;
+    return IA64_NO_FAULT;
 }
 
-IA64FAULT vmx_emul_itc_d(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_itc_d(VCPU *vcpu, INST64 inst)
 {
-    UINT64 itir, ifa, pte;
+    u64 itir, ifa, pte;
 
     if ( itc_fault_check(vcpu, inst, &itir, &ifa, &pte) == IA64_FAULT ) {
     	return IA64_FAULT;
     }
 
-   return (vmx_vcpu_itc_d(vcpu,pte,itir,ifa));
+    debugger_event(XEN_IA64_DEBUG_ON_TC);
+    
+    return vmx_vcpu_itc_d(vcpu, pte, itir, ifa);
 }
 
-IA64FAULT vmx_emul_itc_i(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_itc_i(VCPU *vcpu, INST64 inst)
 {
-    UINT64 itir, ifa, pte;
+    u64 itir, ifa, pte;
 
     if ( itc_fault_check(vcpu, inst, &itir, &ifa, &pte) == IA64_FAULT ) {
     	return IA64_FAULT;
     }
 
-   return (vmx_vcpu_itc_i(vcpu,pte,itir,ifa));
+    debugger_event(XEN_IA64_DEBUG_ON_TC);
 
+    return vmx_vcpu_itc_i(vcpu, pte, itir, ifa);
 }
 
 /*************************************
  * Moves to semi-privileged registers
 *************************************/
 
-IA64FAULT vmx_emul_mov_to_ar_imm(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_ar_imm(VCPU *vcpu, INST64 inst)
 {
     // I27 and M30 are identical for these fields
-    UINT64  imm;
+    u64 imm;
 
     if(inst.M30.ar3!=44){
         panic_domain(vcpu_regs(vcpu),"Can't support ar register other than itc");
@@ -780,7 +830,7 @@ IA64FAULT vmx_emul_mov_to_ar_imm(VCPU *vcpu, INST64 inst)
     return (vmx_vcpu_set_itc(vcpu, imm));
 }
 
-IA64FAULT vmx_emul_mov_to_ar_reg(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_ar_reg(VCPU *vcpu, INST64 inst)
 {
     // I26 and M29 are identical for these fields
     u64 r2;
@@ -808,7 +858,7 @@ IA64FAULT vmx_emul_mov_to_ar_reg(VCPU *vcpu, INST64 inst)
 }
 
 
-IA64FAULT vmx_emul_mov_from_ar_reg(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_from_ar_reg(VCPU *vcpu, INST64 inst)
 {
     // I27 and M30 are identical for these fields
     u64 r1;
@@ -830,7 +880,7 @@ IA64FAULT vmx_emul_mov_from_ar_reg(VCPU *vcpu, INST64 inst)
         return IA64_FAULT;
     }
 #endif // CHECK_FAULT
-    vmx_vcpu_get_itc(vcpu,&r1);
+    r1 = vmx_vcpu_get_itc(vcpu);
     vcpu_set_gr(vcpu,inst.M31.r1,r1,0);
     return IA64_NO_FAULT;
 }
@@ -840,7 +890,7 @@ IA64FAULT vmx_emul_mov_from_ar_reg(VCPU *vcpu, INST64 inst)
  * Moves to privileged registers
 ********************************/
 
-IA64FAULT vmx_emul_mov_to_pkr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_pkr(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r2;
 #ifdef  CHECK_FAULT
@@ -863,7 +913,7 @@ IA64FAULT vmx_emul_mov_to_pkr(VCPU *vcpu, INST64 inst)
     return (vmx_vcpu_set_pkr(vcpu,r3,r2));
 }
 
-IA64FAULT vmx_emul_mov_to_rr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_rr(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r2;
 #ifdef  CHECK_FAULT
@@ -886,10 +936,9 @@ IA64FAULT vmx_emul_mov_to_rr(VCPU *vcpu, INST64 inst)
     return (vmx_vcpu_set_rr(vcpu,r3,r2));
 }
 
-IA64FAULT vmx_emul_mov_to_dbr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_dbr(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r2;
-    return IA64_NO_FAULT;
 #ifdef  CHECK_FAULT
     IA64_PSR vpsr;
     vpsr.val=vmx_vcpu_get_psr(vcpu);
@@ -910,10 +959,9 @@ IA64FAULT vmx_emul_mov_to_dbr(VCPU *vcpu, INST64 inst)
     return (vmx_vcpu_set_dbr(vcpu,r3,r2));
 }
 
-IA64FAULT vmx_emul_mov_to_ibr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_ibr(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r2;
-    return IA64_NO_FAULT;
 #ifdef  CHECK_FAULT
     IA64_PSR vpsr;
     vpsr.val=vmx_vcpu_get_psr(vcpu);
@@ -931,10 +979,10 @@ IA64FAULT vmx_emul_mov_to_ibr(VCPU *vcpu, INST64 inst)
         return IA64_FAULT;
 #endif  //CHECK_FAULT
     }
-    return (vmx_vcpu_set_ibr(vcpu,r3,r2));
+    return vmx_vcpu_set_ibr(vcpu,r3,r2);
 }
 
-IA64FAULT vmx_emul_mov_to_pmc(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_pmc(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r2;
 #ifdef  CHECK_FAULT
@@ -957,7 +1005,7 @@ IA64FAULT vmx_emul_mov_to_pmc(VCPU *vcpu, INST64 inst)
     return (vmx_vcpu_set_pmc(vcpu,r3,r2));
 }
 
-IA64FAULT vmx_emul_mov_to_pmd(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_pmd(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r2;
 #ifdef  CHECK_FAULT
@@ -985,7 +1033,7 @@ IA64FAULT vmx_emul_mov_to_pmd(VCPU *vcpu, INST64 inst)
  * Moves from privileged registers
  **********************************/
 
-IA64FAULT vmx_emul_mov_from_rr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_from_rr(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r1;
 #ifdef  CHECK_FAULT
@@ -1021,7 +1069,7 @@ IA64FAULT vmx_emul_mov_from_rr(VCPU *vcpu, INST64 inst)
     return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
 }
 
-IA64FAULT vmx_emul_mov_from_pkr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_from_pkr(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r1;
 #ifdef  CHECK_FAULT
@@ -1054,11 +1102,91 @@ IA64FAULT vmx_emul_mov_from_pkr(VCPU *vcpu, INST64 inst)
         return IA64_FAULT;
     }
 #endif  //CHECK_FAULT
-    vmx_vcpu_get_pkr(vcpu,r3,&r1);
+    r1 = vmx_vcpu_get_pkr(vcpu, r3);
     return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
 }
 
-IA64FAULT vmx_emul_mov_from_dbr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_from_dbr(VCPU *vcpu, INST64 inst)
+{
+    u64 r3,r1;
+    IA64FAULT res;
+#ifdef  CHECK_FAULT
+    if(check_target_register(vcpu, inst.M43.r1)){
+        set_illegal_op_isr(vcpu);
+        illegal_op(vcpu);
+        return IA64_FAULT;
+    }
+    IA64_PSR vpsr;
+    vpsr.val=vmx_vcpu_get_psr(vcpu);
+    if (vpsr.cpl != 0) {
+        /* Inject Privileged Operation fault into guest */
+        set_privileged_operation_isr (vcpu, 0);
+        privilege_op (vcpu);
+        return IA64_FAULT;
+    }
+
+#endif //CHECK_FAULT
+     if(vcpu_get_gr_nat(vcpu,inst.M43.r3,&r3)){
+#ifdef  CHECK_FAULT
+        set_isr_reg_nat_consumption(vcpu,0,0);
+        rnat_comsumption(vcpu);
+        return IA64_FAULT;
+#endif  //CHECK_FAULT
+    }
+#ifdef  CHECK_FAULT
+    if(is_reserved_indirect_register(vcpu,r3)){
+        set_rsv_reg_field_isr(vcpu);
+        rsv_reg_field(vcpu);
+        return IA64_FAULT;
+    }
+#endif  //CHECK_FAULT
+    res = vmx_vcpu_get_dbr(vcpu, r3, &r1);
+    if (res != IA64_NO_FAULT)
+        return res;
+    return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
+}
+
+static IA64FAULT vmx_emul_mov_from_ibr(VCPU *vcpu, INST64 inst)
+{
+    u64 r3,r1;
+    IA64FAULT res;
+#ifdef  CHECK_FAULT
+    if(check_target_register(vcpu, inst.M43.r1)){
+        set_illegal_op_isr(vcpu);
+        illegal_op(vcpu);
+        return IA64_FAULT;
+    }
+    IA64_PSR vpsr;
+    vpsr.val=vmx_vcpu_get_psr(vcpu);
+    if (vpsr.cpl != 0) {
+        /* Inject Privileged Operation fault into guest */
+        set_privileged_operation_isr (vcpu, 0);
+        privilege_op (vcpu);
+        return IA64_FAULT;
+    }
+
+#endif //CHECK_FAULT
+     if(vcpu_get_gr_nat(vcpu,inst.M43.r3,&r3)){
+#ifdef  CHECK_FAULT
+        set_isr_reg_nat_consumption(vcpu,0,0);
+        rnat_comsumption(vcpu);
+        return IA64_FAULT;
+#endif  //CHECK_FAULT
+    }
+#ifdef  CHECK_FAULT
+    if(is_reserved_indirect_register(vcpu,r3)){
+        set_rsv_reg_field_isr(vcpu);
+        rsv_reg_field(vcpu);
+        return IA64_FAULT;
+    }
+#endif  //CHECK_FAULT
+    res = vmx_vcpu_get_ibr(vcpu, r3, &r1);
+    if (res != IA64_NO_FAULT)
+        return res;
+    return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
+}
+
+static IA64FAULT vmx_emul_mov_from_pmc(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r1;
 #ifdef  CHECK_FAULT
@@ -1091,85 +1219,11 @@ IA64FAULT vmx_emul_mov_from_dbr(VCPU *vcpu, INST64 inst)
         return IA64_FAULT;
     }
 #endif  //CHECK_FAULT
-    vmx_vcpu_get_dbr(vcpu,r3,&r1);
+    r1 = vmx_vcpu_get_pmc(vcpu, r3);
     return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
 }
 
-IA64FAULT vmx_emul_mov_from_ibr(VCPU *vcpu, INST64 inst)
-{
-    u64 r3,r1;
-#ifdef  CHECK_FAULT
-    if(check_target_register(vcpu, inst.M43.r1)){
-        set_illegal_op_isr(vcpu);
-        illegal_op(vcpu);
-        return IA64_FAULT;
-    }
-    IA64_PSR vpsr;
-    vpsr.val=vmx_vcpu_get_psr(vcpu);
-    if (vpsr.cpl != 0) {
-        /* Inject Privileged Operation fault into guest */
-        set_privileged_operation_isr (vcpu, 0);
-        privilege_op (vcpu);
-        return IA64_FAULT;
-    }
-
-#endif //CHECK_FAULT
-     if(vcpu_get_gr_nat(vcpu,inst.M43.r3,&r3)){
-#ifdef  CHECK_FAULT
-        set_isr_reg_nat_consumption(vcpu,0,0);
-        rnat_comsumption(vcpu);
-        return IA64_FAULT;
-#endif  //CHECK_FAULT
-    }
-#ifdef  CHECK_FAULT
-    if(is_reserved_indirect_register(vcpu,r3)){
-        set_rsv_reg_field_isr(vcpu);
-        rsv_reg_field(vcpu);
-        return IA64_FAULT;
-    }
-#endif  //CHECK_FAULT
-    vmx_vcpu_get_ibr(vcpu,r3,&r1);
-    return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
-}
-
-IA64FAULT vmx_emul_mov_from_pmc(VCPU *vcpu, INST64 inst)
-{
-    u64 r3,r1;
-#ifdef  CHECK_FAULT
-    if(check_target_register(vcpu, inst.M43.r1)){
-        set_illegal_op_isr(vcpu);
-        illegal_op(vcpu);
-        return IA64_FAULT;
-    }
-    IA64_PSR vpsr;
-    vpsr.val=vmx_vcpu_get_psr(vcpu);
-    if (vpsr.cpl != 0) {
-        /* Inject Privileged Operation fault into guest */
-        set_privileged_operation_isr (vcpu, 0);
-        privilege_op (vcpu);
-        return IA64_FAULT;
-    }
-
-#endif //CHECK_FAULT
-     if(vcpu_get_gr_nat(vcpu,inst.M43.r3,&r3)){
-#ifdef  CHECK_FAULT
-        set_isr_reg_nat_consumption(vcpu,0,0);
-        rnat_comsumption(vcpu);
-        return IA64_FAULT;
-#endif  //CHECK_FAULT
-    }
-#ifdef  CHECK_FAULT
-    if(is_reserved_indirect_register(vcpu,r3)){
-        set_rsv_reg_field_isr(vcpu);
-        rsv_reg_field(vcpu);
-        return IA64_FAULT;
-    }
-#endif  //CHECK_FAULT
-    vmx_vcpu_get_pmc(vcpu,r3,&r1);
-    return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
-}
-
-IA64FAULT vmx_emul_mov_from_cpuid(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_from_cpuid(VCPU *vcpu, INST64 inst)
 {
     u64 r3,r1;
 #ifdef  CHECK_FAULT
@@ -1193,11 +1247,11 @@ IA64FAULT vmx_emul_mov_from_cpuid(VCPU *vcpu, INST64 inst)
         return IA64_FAULT;
     }
 #endif  //CHECK_FAULT
-    vmx_vcpu_get_cpuid(vcpu,r3,&r1);
+    r1 = vmx_vcpu_get_cpuid(vcpu, r3);
     return vcpu_set_gr(vcpu, inst.M43.r1, r1,0);
 }
 
-IA64FAULT vmx_emul_mov_to_cr(VCPU *vcpu, INST64 inst)
+static IA64FAULT vmx_emul_mov_to_cr(VCPU *vcpu, INST64 inst)
 {
     u64 r2;
     extern u64 cr_igfld_mask(int index, u64 value);
@@ -1234,7 +1288,7 @@ IA64FAULT vmx_emul_mov_to_cr(VCPU *vcpu, INST64 inst)
 #endif  //CHECK_FAULT
     r2 = cr_igfld_mask(inst.M32.cr3,r2);
     switch (inst.M32.cr3) {
-        case 0: return vmx_vcpu_set_dcr(vcpu,r2);
+        case 0: return vcpu_set_dcr(vcpu,r2);
         case 1: return vmx_vcpu_set_itm(vcpu,r2);
         case 2: return vmx_vcpu_set_iva(vcpu,r2);
         case 8: return vmx_vcpu_set_pta(vcpu,r2);
@@ -1271,14 +1325,20 @@ IA64FAULT vmx_emul_mov_to_cr(VCPU *vcpu, INST64 inst)
     ((fault=vcpu_get_##cr(vcpu,&val))==IA64_NO_FAULT)?\
         vcpu_set_gr(vcpu, tgt, val,0):fault;
 
+//#define cr_get(cr) (vcpu_set_gr(vcpu, tgt, vcpu_get##cr(vcpu), 0)
+
+/*
 #define vmx_cr_get(cr) \
     ((fault=vmx_vcpu_get_##cr(vcpu,&val))==IA64_NO_FAULT)?\
         vcpu_set_gr(vcpu, tgt, val,0):fault;
+*/
 
-IA64FAULT vmx_emul_mov_from_cr(VCPU *vcpu, INST64 inst)
+#define vmx_cr_get(cr) (vcpu_set_gr(vcpu, tgt, vmx_vcpu_get_##cr(vcpu), 0))
+
+static IA64FAULT vmx_emul_mov_from_cr(VCPU *vcpu, INST64 inst)
 {
-    UINT64 tgt = inst.M33.r1;
-    UINT64 val;
+    u64 tgt = inst.M33.r1;
+    u64 val;
     IA64FAULT fault;
 #ifdef  CHECK_FAULT
     IA64_PSR vpsr;
@@ -1299,7 +1359,7 @@ IA64FAULT vmx_emul_mov_from_cr(VCPU *vcpu, INST64 inst)
 
 //    from_cr_cnt[inst.M33.cr3]++;
     switch (inst.M33.cr3) {
-        case 0: return vmx_cr_get(dcr);
+        case 0: return cr_get(dcr);
         case 1: return vmx_cr_get(itm);
         case 2: return vmx_cr_get(iva);
         case 8: return vmx_cr_get(pta);
@@ -1314,7 +1374,7 @@ IA64FAULT vmx_emul_mov_from_cr(VCPU *vcpu, INST64 inst)
         case 25:return cr_get(iha);
         case 64:return vmx_cr_get(lid);
         case 65:
-                vmx_vcpu_get_ivr(vcpu,&val);
+                val = vmx_vcpu_get_ivr(vcpu);
                 return vcpu_set_gr(vcpu,tgt,val,0);
         case 66:return vmx_cr_get(tpr);
         case 67:return vcpu_set_gr(vcpu,tgt,0L,0);
@@ -1353,7 +1413,7 @@ vmx_emulate(VCPU *vcpu, REGS *regs)
 {
     IA64FAULT status;
     INST64 inst;
-    UINT64 iip, cause, opcode;
+    u64 iip, cause, opcode;
     iip = regs->cr_iip;
     cause = VMX(vcpu,cause);
     opcode = VMX(vcpu,opcode);
@@ -1364,7 +1424,7 @@ vmx_emulate(VCPU *vcpu, REGS *regs)
 #endif
 #if 0
 if ( (cause == 0xff && opcode == 0x1e000000000) || cause == 0 ) {
-		printf ("VMAL decode error: cause - %lx; op - %lx\n", 
+		printk ("VMAL decode error: cause - %lx; op - %lx\n", 
 			cause, opcode );
 		return;
 }
@@ -1381,7 +1441,7 @@ if ( (cause == 0xff && opcode == 0x1e000000000) || cause == 0 ) {
     else if (slot == 1)
         inst.inst = bundle.slot1a + (bundle.slot1b<<18);
     else if (slot == 2) inst.inst = bundle.slot2;
-    else printf("priv_handle_op: illegal slot: %d\n", slot);
+    else printk("priv_handle_op: illegal slot: %d\n", slot);
     slot_type = slot_types[bundle.template][slot];
     ia64_priv_decoder(slot_type, inst, &cause);
     if(cause==0){
@@ -1390,6 +1450,9 @@ if ( (cause == 0xff && opcode == 0x1e000000000) || cause == 0 ) {
 #else
     inst.inst=opcode;
 #endif /* BYPASS_VMAL_OPCODE */
+
+    debugger_event(XEN_IA64_DEBUG_ON_PRIVOP);
+
     /*
      * Switch to actual virtual rid in rr0 and rr4,
      * which is required by some tlb related instructions.
@@ -1398,179 +1461,195 @@ if ( (cause == 0xff && opcode == 0x1e000000000) || cause == 0 ) {
 
     switch(cause) {
     case EVENT_RSM:
-        perfc_incrc(vmx_rsm);
+        perfc_incr(vmx_rsm);
         status=vmx_emul_rsm(vcpu, inst);
         break;
     case EVENT_SSM:
-        perfc_incrc(vmx_ssm);
+        perfc_incr(vmx_ssm);
         status=vmx_emul_ssm(vcpu, inst);
         break;
     case EVENT_MOV_TO_PSR:
-        perfc_incrc(vmx_mov_to_psr);
+        perfc_incr(vmx_mov_to_psr);
         status=vmx_emul_mov_to_psr(vcpu, inst);
         break;
     case EVENT_MOV_FROM_PSR:
-        perfc_incrc(vmx_mov_from_psr);
+        perfc_incr(vmx_mov_from_psr);
         status=vmx_emul_mov_from_psr(vcpu, inst);
         break;
     case EVENT_MOV_FROM_CR:
-        perfc_incrc(vmx_mov_from_cr);
+        perfc_incr(vmx_mov_from_cr);
         status=vmx_emul_mov_from_cr(vcpu, inst);
         break;
     case EVENT_MOV_TO_CR:
-        perfc_incrc(vmx_mov_to_cr);
+        perfc_incr(vmx_mov_to_cr);
         status=vmx_emul_mov_to_cr(vcpu, inst);
         break;
     case EVENT_BSW_0:
-        perfc_incrc(vmx_bsw0);
+        perfc_incr(vmx_bsw0);
         status=vmx_emul_bsw0(vcpu, inst);
         break;
     case EVENT_BSW_1:
-        perfc_incrc(vmx_bsw1);
+        perfc_incr(vmx_bsw1);
         status=vmx_emul_bsw1(vcpu, inst);
         break;
     case EVENT_COVER:
-        perfc_incrc(vmx_cover);
+        perfc_incr(vmx_cover);
         status=vmx_emul_cover(vcpu, inst);
         break;
     case EVENT_RFI:
-        perfc_incrc(vmx_rfi);
+        perfc_incr(vmx_rfi);
         status=vmx_emul_rfi(vcpu, inst);
         break;
     case EVENT_ITR_D:
-        perfc_incrc(vmx_itr_d);
+        perfc_incr(vmx_itr_d);
         status=vmx_emul_itr_d(vcpu, inst);
         break;
     case EVENT_ITR_I:
-        perfc_incrc(vmx_itr_i);
+        perfc_incr(vmx_itr_i);
         status=vmx_emul_itr_i(vcpu, inst);
         break;
     case EVENT_PTR_D:
-        perfc_incrc(vmx_ptr_d);
+        perfc_incr(vmx_ptr_d);
         status=vmx_emul_ptr_d(vcpu, inst);
         break;
     case EVENT_PTR_I:
-        perfc_incrc(vmx_ptr_i);
+        perfc_incr(vmx_ptr_i);
         status=vmx_emul_ptr_i(vcpu, inst);
         break;
     case EVENT_ITC_D:
-        perfc_incrc(vmx_itc_d);
+        perfc_incr(vmx_itc_d);
         status=vmx_emul_itc_d(vcpu, inst);
         break;
     case EVENT_ITC_I:
-        perfc_incrc(vmx_itc_i);
+        perfc_incr(vmx_itc_i);
         status=vmx_emul_itc_i(vcpu, inst);
         break;
     case EVENT_PTC_L:
-        perfc_incrc(vmx_ptc_l);
+        perfc_incr(vmx_ptc_l);
         status=vmx_emul_ptc_l(vcpu, inst);
         break;
     case EVENT_PTC_G:
-        perfc_incrc(vmx_ptc_g);
+        perfc_incr(vmx_ptc_g);
         status=vmx_emul_ptc_g(vcpu, inst);
         break;
     case EVENT_PTC_GA:
-        perfc_incrc(vmx_ptc_ga);
+        perfc_incr(vmx_ptc_ga);
         status=vmx_emul_ptc_ga(vcpu, inst);
         break;
     case EVENT_PTC_E:
-        perfc_incrc(vmx_ptc_e);
+        perfc_incr(vmx_ptc_e);
         status=vmx_emul_ptc_e(vcpu, inst);
         break;
     case EVENT_MOV_TO_RR:
-        perfc_incrc(vmx_mov_to_rr);
+        perfc_incr(vmx_mov_to_rr);
         status=vmx_emul_mov_to_rr(vcpu, inst);
         break;
     case EVENT_MOV_FROM_RR:
-        perfc_incrc(vmx_mov_from_rr);
+        perfc_incr(vmx_mov_from_rr);
         status=vmx_emul_mov_from_rr(vcpu, inst);
         break;
     case EVENT_THASH:
-        perfc_incrc(vmx_thash);
+        perfc_incr(vmx_thash);
         status=vmx_emul_thash(vcpu, inst);
         break;
     case EVENT_TTAG:
-        perfc_incrc(vmx_ttag);
+        perfc_incr(vmx_ttag);
         status=vmx_emul_ttag(vcpu, inst);
         break;
     case EVENT_TPA:
-        perfc_incrc(vmx_tpa);
+        perfc_incr(vmx_tpa);
         status=vmx_emul_tpa(vcpu, inst);
         break;
     case EVENT_TAK:
-        perfc_incrc(vmx_tak);
+        perfc_incr(vmx_tak);
         status=vmx_emul_tak(vcpu, inst);
         break;
     case EVENT_MOV_TO_AR_IMM:
-        perfc_incrc(vmx_mov_to_ar_imm);
+        perfc_incr(vmx_mov_to_ar_imm);
         status=vmx_emul_mov_to_ar_imm(vcpu, inst);
         break;
     case EVENT_MOV_TO_AR:
-        perfc_incrc(vmx_mov_to_ar_reg);
+        perfc_incr(vmx_mov_to_ar_reg);
         status=vmx_emul_mov_to_ar_reg(vcpu, inst);
         break;
     case EVENT_MOV_FROM_AR:
-        perfc_incrc(vmx_mov_from_ar_reg);
+        perfc_incr(vmx_mov_from_ar_reg);
         status=vmx_emul_mov_from_ar_reg(vcpu, inst);
         break;
     case EVENT_MOV_TO_DBR:
-        perfc_incrc(vmx_mov_to_dbr);
+        perfc_incr(vmx_mov_to_dbr);
         status=vmx_emul_mov_to_dbr(vcpu, inst);
         break;
     case EVENT_MOV_TO_IBR:
-        perfc_incrc(vmx_mov_to_ibr);
+        perfc_incr(vmx_mov_to_ibr);
         status=vmx_emul_mov_to_ibr(vcpu, inst);
         break;
     case EVENT_MOV_TO_PMC:
-        perfc_incrc(vmx_mov_to_pmc);
+        perfc_incr(vmx_mov_to_pmc);
         status=vmx_emul_mov_to_pmc(vcpu, inst);
         break;
     case EVENT_MOV_TO_PMD:
-        perfc_incrc(vmx_mov_to_pmd);
+        perfc_incr(vmx_mov_to_pmd);
         status=vmx_emul_mov_to_pmd(vcpu, inst);
         break;
     case EVENT_MOV_TO_PKR:
-        perfc_incrc(vmx_mov_to_pkr);
+        perfc_incr(vmx_mov_to_pkr);
         status=vmx_emul_mov_to_pkr(vcpu, inst);
         break;
     case EVENT_MOV_FROM_DBR:
-        perfc_incrc(vmx_mov_from_dbr);
+        perfc_incr(vmx_mov_from_dbr);
         status=vmx_emul_mov_from_dbr(vcpu, inst);
         break;
     case EVENT_MOV_FROM_IBR:
-        perfc_incrc(vmx_mov_from_ibr);
+        perfc_incr(vmx_mov_from_ibr);
         status=vmx_emul_mov_from_ibr(vcpu, inst);
         break;
     case EVENT_MOV_FROM_PMC:
-        perfc_incrc(vmx_mov_from_pmc);
+        perfc_incr(vmx_mov_from_pmc);
         status=vmx_emul_mov_from_pmc(vcpu, inst);
         break;
     case EVENT_MOV_FROM_PKR:
-        perfc_incrc(vmx_mov_from_pkr);
+        perfc_incr(vmx_mov_from_pkr);
         status=vmx_emul_mov_from_pkr(vcpu, inst);
         break;
     case EVENT_MOV_FROM_CPUID:
-        perfc_incrc(vmx_mov_from_cpuid);
+        perfc_incr(vmx_mov_from_cpuid);
         status=vmx_emul_mov_from_cpuid(vcpu, inst);
         break;
     case EVENT_VMSW:
-        printf ("Unimplemented instruction %ld\n", cause);
-	status=IA64_FAULT;
+        printk ("Unimplemented instruction %ld\n", cause);
+        status=IA64_FAULT;
         break;
     default:
-        panic_domain(regs,"unknown cause %ld, iip: %lx, ipsr: %lx\n", cause,regs->cr_iip,regs->cr_ipsr);
+        panic_domain(regs,"unknown cause %ld, iip: %lx, ipsr: %lx\n",
+                     cause,regs->cr_iip,regs->cr_ipsr);
         break;
     };
 
 #if 0
-    if (status == IA64_FAULT)
+    if (status != IA64_NO_FAULT)
 	panic("Emulation failed with cause %d:\n", cause);
 #endif
 
-    if ( status == IA64_NO_FAULT && cause !=EVENT_RFI ) {
-        vmx_vcpu_increment_iip(vcpu);
+    switch (status) {
+    case IA64_RSVDREG_FAULT:
+        set_rsv_reg_field_isr(vcpu);
+        rsv_reg_field(vcpu);
+        break;
+    case IA64_ILLOP_FAULT:
+        set_illegal_op_isr(vcpu);
+        illegal_op(vcpu);
+        break;
+    case IA64_FAULT:
+        /* Registers aleady set.  */
+        break;
+    case IA64_NO_FAULT:
+        if ( cause != EVENT_RFI )
+            vcpu_increment_iip(vcpu);
+        break;
     }
 
+
     recover_if_physical_mode(vcpu);
     return;
 
