@@ -21,7 +21,6 @@
 
 #include <asm/config.h>
 #include <asm/hvm/io.h>
-#include <asm/hvm/vmx/cpu.h>
 
 #ifdef VMXASSIST
 #include <public/hvm/vmx_assist.h>
@@ -89,10 +88,20 @@ struct arch_vmx_struct {
     struct vmx_msr_entry *host_msr_area;
 
 #ifdef VMXASSIST
+
     unsigned long        vmxassist_enabled:1;
     unsigned long        irqbase_mode:1;
     unsigned char        pm_irqbase[2];
+
 #else
+
+    /* Are we emulating rather than VMENTERing? */
+#define VMXEMUL_REALMODE 1  /* Yes, because CR0.PE == 0   */
+#define VMXEMUL_BAD_CS   2  /* Yes, because CS.RPL != CPL */
+#define VMXEMUL_BAD_SS   4  /* Yes, because SS.RPL != CPL */
+    uint8_t              vmxemul;
+
+    /* I/O request in flight to device model. */
     bool_t               real_mode_io_in_progress;
     bool_t               real_mode_io_completed;
     unsigned long        real_mode_io_data;
@@ -111,6 +120,8 @@ void vmx_vmcs_exit(struct vcpu *v);
 #define CPU_BASED_MWAIT_EXITING               0x00000400
 #define CPU_BASED_RDPMC_EXITING               0x00000800
 #define CPU_BASED_RDTSC_EXITING               0x00001000
+#define CPU_BASED_CR3_LOAD_EXITING            0x00008000
+#define CPU_BASED_CR3_STORE_EXITING           0x00010000
 #define CPU_BASED_CR8_LOAD_EXITING            0x00080000
 #define CPU_BASED_CR8_STORE_EXITING           0x00100000
 #define CPU_BASED_TPR_SHADOW                  0x00200000
