@@ -16,7 +16,6 @@ struct schedule_data {
     struct vcpu        *idle;           /* idle task for this cpu          */
     void               *sched_priv;
     struct timer        s_timer;        /* scheduling timer                */
-    unsigned long       tick;           /* current periodic 'tick'         */
 } __cacheline_aligned;
 
 DECLARE_PER_CPU(struct schedule_data, schedule_data);
@@ -61,16 +60,19 @@ struct scheduler {
     unsigned int sched_id;  /* ID for this scheduler             */
 
     void         (*init)           (void);
-    void         (*tick)           (unsigned int cpu);
+
+    int          (*init_domain)    (struct domain *);
+    void         (*destroy_domain) (struct domain *);
 
     int          (*init_vcpu)      (struct vcpu *);
-    void         (*destroy_domain) (struct domain *);
+    void         (*destroy_vcpu)   (struct vcpu *);
 
     void         (*sleep)          (struct vcpu *);
     void         (*wake)           (struct vcpu *);
 
     struct task_slice (*do_schedule) (s_time_t);
 
+    int          (*pick_cpu)       (struct vcpu *);
     int          (*adjust)         (struct domain *,
                                     struct xen_domctl_scheduler_op *);
     void         (*dump_settings)  (void);
