@@ -8,9 +8,14 @@
 #define KEYPRESS_SOFTIRQ                  3
 #define NMI_SOFTIRQ                       4
 #define PAGE_SCRUB_SOFTIRQ                5
-#define DOMAIN_SHUTDOWN_FINALISE_SOFTIRQ  6
-#define TRACE_SOFTIRQ                     7
-#define NR_SOFTIRQS                       8
+#define TRACE_SOFTIRQ                     6
+#define RCU_SOFTIRQ                       7
+
+#define NR_COMMON_SOFTIRQS                8
+
+#include <asm/softirq.h>
+
+#define NR_SOFTIRQS (NR_COMMON_SOFTIRQS + NR_ARCH_SOFTIRQS)
 
 #ifndef __ASSEMBLY__
 
