@@ -1448,8 +1448,9 @@ inchr c;
       }
     else {
       for (k=0;k<smushamount;k++) {
-        outputline[row][outlinelen-smushamount+k] =
-          smushem(outputline[row][outlinelen-smushamount+k],currchar[row][k]);
+        if (outlinelen-smushamount+k >= 0)
+          outputline[row][outlinelen-smushamount+k] =
+            smushem(outputline[row][outlinelen-smushamount+k],currchar[row][k]);
         }
       strcat(outputline[row],currchar[row]+smushamount);
       }
@@ -1487,7 +1488,16 @@ static void myputchar(unsigned char c)
 
     putc(c, stderr);
 
-    if ( nr_chars == 0 )
+    if ( nr_chars == 18 ) 
+    {
+        nr_chars = 0;
+        putchar('"');
+        putchar(' ');
+        putchar('\\');
+        putchar('\n');
+    }
+
+    if ( nr_chars++ == 0 )
         putchar('"');
 
     putchar('\\');
@@ -1497,15 +1507,6 @@ static void myputchar(unsigned char c)
 
     if ( c == '\n' )
         startline = 1;
-
-    if ( ++nr_chars == 18 ) 
-    {
-        nr_chars = 0;
-        putchar('"');
-        putchar(' ');
-        putchar('\\');
-        putchar('\n');
-    }
 }
 
 void putstring(string)
